//----------------------------------------//
// camera.cpp - main file for the camera class/functions
//
//----------------------------------------//

#include "camera.h"

//----------------------------------------//
// Name: CAMERA2()
// Purpose: constructor - gives default values
//----------------------------------------//
CAMERA::CAMERA()
{
	//CameraState = FIRSTPERSON;
	FarClip = 1000.0f;
	Fov = D3DX_PI/4;
	LookAt = D3DXVECTOR3(2.0f, 2.0f, 1.0f);
	D3DXMatrixIdentity(&matProjection);
	D3DXMatrixIdentity(&matView);
	NearClip = 0.05f;
	Pitch = 0.0f;
	PitchVelocity = 0.0f;
	Position = D3DXVECTOR3(2.0f, 2.0f, 0.0f);
	Velocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	Yaw = 0.0f;
	YawVelocity = 0.0f;

}


//----------------------------------------//
// Name: InitializeCamera()
// Purpose: initializes the camera to the set values
//----------------------------------------//
int CAMERA::InitializeCamera(LPDIRECT3DDEVICE8 device, int screenwidth, int screenheight)
{

	//set up the view matrix
	D3DXMatrixLookAtLH(&matView, &D3DXVECTOR3(Position.x, Position.y, Position.z),	//camera position
                                  &D3DXVECTOR3(LookAt.x, LookAt.y, LookAt.z),	//lookat position
                                  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );	//up vector
	//set the view projection
	device->SetTransform(D3DTS_VIEW, &matView);

	//set the projection matrix
	D3DXMatrixPerspectiveFovLH(&matProjection, Fov, float(screenwidth/screenheight), 
										NearClip, FarClip);
	device->SetTransform(D3DTS_PROJECTION, &matProjection);

	//return ok
	return CAMERA_OK;
}


//----------------------------------------//
// Name: RotateCamera()
// Purpose: rotates the camera on the x and y axis
//----------------------------------------//
void CAMERA::RotateCamera(float yaw, float pitch)
{
	yaw/=15.0f;
	pitch/=15.0f;

	//simply add to the velocity
	PitchVelocity+=pitch;
	YawVelocity +=yaw;

	/*if (Pitch <= 0.01f) 
	{
		Pitch = 0.01f;
		PitchVelocity = 0.0f;
	}

	if (Pitch >= 3.13f) 
	{
		Pitch = 3.13f;
		PitchVelocity = 0.0f;
	}*/

}


//----------------------------------------//
// Name: TranslateCamera()
// Purpose: moves the camera with the correct speed according to what way
//		the camera is facing - z isn't implemented yet *** FIX ME ***
//----------------------------------------//
void CAMERA::TranslateCamera(float x, float y, float z)
{
	Velocity.x += x*cosf(-Yaw) - z*sinf(-Yaw);
	Velocity.z += x*sinf(-Yaw) + z*cosf(-Yaw);

}


//----------------------------------------//
// Name: UpdateCamera()
// Purpose: updates the camera projections/matrices
//----------------------------------------//
void CAMERA::UpdateCamera(LPDIRECT3DDEVICE8 device, float delta)
{



	Position += Velocity*delta;
	Yaw += YawVelocity*delta;
	Pitch += PitchVelocity*delta;

	Velocity /= 1.2f;
	YawVelocity /= 1.2f;
	PitchVelocity /= 1.2f;

	// calculate lookat on the fly.
	// this should produce a vector that is slightly
	// in front of the camera no matter where the camera is.

	/*LookAt = D3DXVECTOR3(sinf(Yaw)*cosf(Pitch) + Position.x,
							sinf(Pitch),
							cosf(Yaw)*cosf(Pitch) + Position.z);*/
	LookAt.x = (sinf(Yaw)*cosf(Pitch)) + Position.x;
	LookAt.y = sinf(-Pitch) + Position.y;
	LookAt.z = (cosf(Yaw)*cosf(Pitch)) + Position.z;

	D3DXMatrixLookAtLH(&matView, &Position,					//position vector
									&LookAt,						//lookat vector	
									&D3DXVECTOR3(0.0f, 1.0f, 0.0f));	//up vector

	 //set the new matrix
	 device->SetTransform(D3DTS_VIEW, &matView);

}