//----------------------------------------//
// input.cpp - main file for input functions
//
//----------------------------------------//

//include the necessary files
#include "input.h"

//----------------------------------------//
// Name: DI()
// Purpose: the DI constructor
//----------------------------------------//
DI::DI()
{
	//set all the variables to default values
	active = true;
	lpdi = NULL;
	lpdikey = NULL;
	lpdimouse = NULL;
	MouseMultiplier = 1.0f;
	MouseX = 0;
	MouseY = 0;
}

//----------------------------------------//
// Name: ~DI()
// Purpose: the DI destructor
//----------------------------------------//
DI::~DI()
{
	//kill everything *** temp ***
	//ShutdownDirectInput();

}

//----------------------------------------//
// Name: InitDirectInput()
// Purpose: Initializes the mouse and keyboard
//			of Direct Input
//----------------------------------------//
int DI::InitializeDirectInput(HINSTANCE hInst, HWND hWnd)
{
	logfile.LogtoFile("DI - InitDirectInput() called\n");

	//start by creating the input thingy - log results to file
	if(FAILED(DirectInput8Create(hInst, DIRECTINPUT_VERSION,
					IID_IDirectInput8, (void**)&lpdi, NULL)))
	{
		logfile.LogtoFile("DI - Failed to Create a Direct Input interface \n");
		return INPUT_BAD;
	} else 
		logfile.LogtoFile("DI - Direct Input interface successfully created \n");
	
	//initialize the keyboard...

	//create the keyboard device
	if(FAILED(lpdi->CreateDevice(GUID_SysKeyboard, &lpdikey, NULL)))
	{
		logfile.LogtoFile("DI - Failed to Create a Direct Input keyboard device \n");
		return INPUT_BAD;
	} else 
		logfile.LogtoFile("DI - Direct Input Keyboard device created \n");
	
	//set the data format to the keyboard
	if(FAILED(lpdikey->SetDataFormat(&c_dfDIKeyboard)))
	{
		logfile.LogtoFile("DI - Failed to set data format for keyboard \n");
		return INPUT_BAD;
	} else 
		logfile.LogtoFile("DI - Data format set for keyboard \n");

	//set the cooperation level
	if(FAILED(lpdikey->SetCooperativeLevel(hWnd, 
		DISCL_FOREGROUND | DISCL_NONEXCLUSIVE)))
	{
		logfile.LogtoFile("DI - Failed to set cooperative level for keyboard \n");
		return E_FAIL;
	} else
		logfile.LogtoFile("DI - Cooperateive level set for keyboard \n");

	//acquire the device
	if(FAILED(lpdikey->Acquire()))
	{
		logfile.LogtoFile("DI - Error acquiring the keyboard device \n");
		return INPUT_BAD;
	} else
		logfile.LogtoFile("DI - Keyboard acquired \n");

	//initialize the mouse - basically the same as the keyboard

	//create device for mouse
	if(FAILED(lpdi->CreateDevice(GUID_SysMouse, &lpdimouse, NULL)))
	{
		//***temp***
		logfile.LogtoFile("DI - Error creating a mouse device \n");
		return INPUT_BAD;
	} else
		//***temp***
		logfile.LogtoFile("DI - Direct Input Mouse device created \n");

	//set the data format on the mouse
	if(FAILED(lpdimouse->SetDataFormat(&c_dfDIMouse2)))
	{
		logfile.LogtoFile("DI - Error setting mouse data format \n");
		return INPUT_BAD;
	} else
		logfile.LogtoFile("DI - Mouse data format set \n");

	//set the mouse cooperative mode level
	if(FAILED(lpdimouse->SetCooperativeLevel(hWnd, 
		DISCL_FOREGROUND | DISCL_NONEXCLUSIVE)))
	{
		logfile.LogtoFile("DI - Error setting mouse cooperative level \n");
		return INPUT_BAD;
	} else
		logfile.LogtoFile("DI - Cooperative level set for mouse \n");

	//acquire the mouse
	if(FAILED(lpdimouse->Acquire()))
	{
		logfile.LogtoFile("DI - Error acquiring the mouse \n");
		return INPUT_BAD;
	} else
		logfile.LogtoFile("DI - Mouse acquired\n");

	//tell the logfile that all was well
	logfile.LogtoFile("DI - Direct Input initialization succesful.\n");

	//return ok
	return INPUT_OK;

}


//----------------------------------------//
// Name: ShutdownDirectInput()
// Purpose: releases all the stuff for DI
//----------------------------------------//
int DI::ShutdownDirectInput()
{
	//kill the logfile, tell it first
	logfile.LogtoFile("DI - ShutdownDirectInput() called.\n");

	//release the mouse
	if(lpdimouse)
	{
		lpdimouse->Unacquire();
		lpdimouse->Release();
		lpdimouse = NULL;
	}

	//release the keyboard
	if(lpdikey)
	{
		lpdikey->Unacquire();
		lpdikey->Release();
		lpdikey = NULL;
	}

	//release the pointer to Direct Input
	if(lpdi)
	{
		lpdi->Release();
		lpdi = NULL;
	}

	//return ok
	return INPUT_OK;
}


//----------------------------------------//
// Name: ActivateKeyboard();
// Purpose: activates the keyboard - such as
//		if the window goes inactive and is
//		re-activated
//----------------------------------------//
int DI::ActivateKeyboard()
{
	active = true;

	// return success
	return(INPUT_OK);
}


//----------------------------------------//
// Name: DeActivateKeyboard();
// Purpose: deactivates the keyboard - such as
//		if the window goes inactive
//----------------------------------------//
int DI::DeActivateKeyboard()
{
	active = false;

	// return success
	return(INPUT_OK);
}

//--------------------------------------------------//
// Name: KeyDown()
// Desc: tests for a key press
//--------------------------------------------------//
bool DI::KeyDown(const int key)
{
	return((keystate[key] & 0x80)!=0?true:false);
}

//--------------------------------------------------//
// Name: KeyUp()
// Desc: tests for a key release
//--------------------------------------------------//
//function that says a key is up
bool DI::KeyUp(const int key)
{
	return((keystate[key] & 0x80)!=0?false:true);

}

//--------------------------------------------------//
// Name: ReadKeys()
// Desc: reads the keys using DI
//--------------------------------------------------//
int DI::ReadKeys()
{
	HRESULT hr;
	
	//if the keyboard is active...
	if(active == true)
	{
		hr = lpdikey->GetDeviceState(256*sizeof(char),keystate);

		if(FAILED(hr))
		{
			if(hr == DIERR_INPUTLOST ||
				hr == DIERR_NOTACQUIRED)
			{
				hr = lpdikey->Acquire();
				if(FAILED(hr))
				{
					if(hr==DIERR_OTHERAPPHASPRIO)
						return INPUT_BAD;
				}
				lpdikey->GetDeviceState(256*sizeof(char), keystate);
			} else

			return INPUT_BAD;

		}
	
	}

	return INPUT_OK;

}

//--------------------------------------------------//
// Name: ReadMouse()
// Desc: reads the mouse using DI
//--------------------------------------------------//
int DI::ReadMouse()
{
	HRESULT hr;

	//test to see if the mouse is active first, if not return 0
	if(active == true)
	{

		hr = lpdimouse->GetDeviceState(sizeof(DIMOUSESTATE2),
			&mousestate);

		if(FAILED(hr))
		{
			if(hr == DIERR_INPUTLOST)
			{
				//try again to acquire the mouse
				hr = lpdimouse->Acquire();
				if(FAILED(hr))
				{
					//some other app has it
					if(hr==DIERR_OTHERAPPHASPRIO)
						return INPUT_BAD;
				}
				
				//get the data
				lpdimouse->GetDeviceState(sizeof(DIMOUSESTATE2),&mousestate);
			}else

			return INPUT_BAD;
		}
	}

	return INPUT_OK;

}

